/*
 * Copyright: 2005 Axis Communications AB
 * 
 * This work is licensed under the terms of the GNU GPL, version 2 or later.
 * See the COPYING file in the top-level directory.
 * 
 * dhcpv6 lease handling.
 * 
 * Authors:          Edgar E. Iglesias <edgar@axis.com>
 */

struct lease_t {
	time_t aquired;
	int needsreconfig;
	unsigned int raflags;
	unsigned int state;
	struct dhcpv6_option_ia_t *ia;
	struct dhcpv6_nodeid_t *server;
	struct in6_addr addr;
	struct in6_addr oldaddr;
	struct in6_addr syncedaddr;
	struct optionmap_t omap[256];
};

static inline void lease_init(struct lease_t *lease)
{
	memset(lease, 0, sizeof *lease);
}

extern void lease_readfile(char *filename, struct lease_t *lease);
extern void lease_sync(char *filename, struct lease_t *lease);
extern int lease_drop(struct lease_t *lease);
extern void lease_cleanup(struct lease_t *lease);
extern int lease_test_and_clear_reconfig(struct lease_t *lease);
extern void lease_update_oldvalues(struct lease_t *lease);
extern void lease_create_environ(struct lease_t *lease);
extern void lease_set_server(struct lease_t *lz,
                             struct dhcpv6_nodeid_t *server);
extern void lease_set_ia(struct lease_t *lz, struct dhcpv6_option_ia_t *ia);
