/*
 * Copyright: 2005 Axis Communications AB
 * 
 * This work is licensed under the terms of the GNU GPL, version 2 or later.
 * See the COPYING file in the top-level directory.
 * 
 * message buffers, keep this small.
 *
 * Authors:          Edgar E. Iglesias <edgar@axis.com>
 */
#include <assert.h>

struct msgbuf_t {
        size_t pos;
        size_t size;
        unsigned char buf[1];
};

extern struct msgbuf_t * msgbuf_new(size_t size)
__attribute__ ((__warn_unused_result__));

static inline void msgbuf_free(struct msgbuf_t *msg) {
	free(msg);
}

static inline void msgbuf_reset(struct msgbuf_t **msg) {
	assert(msg && *msg);
	(*msg)->pos = 0;
}

static inline size_t msgbuf_save(struct msgbuf_t **msg) {
	assert(msg && *msg);
	return (*msg)->pos;
}

static inline void msgbuf_restore(struct msgbuf_t **msg, size_t savedpos) {
	assert(msg && *msg);
	(*msg)->pos = savedpos;
}

extern size_t msgbuf_save(struct msgbuf_t **msg)
__attribute__ ((__const__))
__attribute__ ((__warn_unused_result__));

extern void msgbuf_append(struct msgbuf_t **msg, const void *buf, size_t len);
extern void msgbuf_append_u8(struct msgbuf_t **msg, uint8_t value);
extern void msgbuf_append_u16no(struct msgbuf_t **msg, uint16_t value);
extern void msgbuf_append_u24no(struct msgbuf_t **msg, uint32_t value);
extern void msgbuf_append_u32no(struct msgbuf_t **msg, uint32_t value);
