/*
 * Copyright: 2007 Axis Communications AB
 * 
 * This work is licensed under the terms of the GNU GPL, version 2 or later.
 * See the COPYING file in the top-level directory.
 *
 * Notifies the distribution about new DHCPv6 options.
 *
 * Authors:	     Edgar E. Iglesias <edgar@axis.com>
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#if HAVE_STDINT_H
#include <stdint.h>
#endif
#if HAVE_INTTYPES_H
#include <inttypes.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#define __GNU_SOURCE
#define __USE_GNU
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <assert.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "optionmap.h"
#include "lease.h"

#undef D
#define D(x)

/* notify via a script or program */
void notify_exec_script(char *script, struct lease_t *lease) {
	pid_t kid;
	int status;
	extern char **environ;

	assert(lease);

	if (!script)
		return;
	
	D(printf("%s %s\n", __func__, script));
	
	kid = fork();
	if (!kid) {
		char *a[2];
		a[0] = script;
		a[1] = NULL;
		
		lease_create_environ(lease);
		
		execve(script, a, environ);
		perror(script);
	}
	else {
		waitpid(kid, &status, 0);
		lease_update_oldvalues(lease);
	}
}
