/*
 * Copyright: 2005 Axis Communications AB
 * 
 * This work is licensed under the terms of the GNU GPL, version 2 or later.
 * See the COPYING file in the top-level directory.
 * 
 * implements a map of options.
 *
 * Authors:          Edgar E. Iglesias <edgar@axis.com>
 */

struct optionmap_t {
	const char *env;
	void  (*update)(struct optionmap_t *o, void *newvalue);
	char       *value;
	char       *oldvalue;
	int         needsreconfig;
};

extern int optionmap_drop(struct optionmap_t *o, size_t optionlen);
extern void optionmap_cleanup(struct optionmap_t *o, size_t optionlen);
extern int optionmap_test_and_clear_reconfig(struct optionmap_t *o,
					     size_t optionlen);
extern void optionmap_create_environ(struct optionmap_t *o, size_t optionlen);
extern void optionmap_update_oldvalues(struct optionmap_t *o,
				       size_t optionlen);

static inline void
optionmap_add(struct optionmap_t *o, int code, char *env,
	      void (*update)(struct optionmap_t *o, void *nv)) {
	o[code].env = env;
	o[code].update = update;
}

static inline
void optionmap_up(struct optionmap_t *o, int code, void *newvalue) {
	if (o[code].update)
		o[code].update(&o[code], newvalue);
}
